<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\{ DateUtil, Environment, NovoCodigo, Repositories };

use Ems\Services\{ EstruturaService, EmailService };
use Ems\Repository\{ SolicitacaoRepository, EscalaRepository, OcupacaoRepository, MovimentacaoRepository, ValeTransporteRepository, FeriasRepository, AlteSalarioRepository, MinhaHistoriaRepository, VinculoRepository, PeriodoRepository };

use Ems\Util\{ ArrayUtil, Service, Utilities, ExpectedException, Utils };

use Ems\Logic\ProcessamentoSolicitacao\{ ProcessamentoSolicitacao, ProcessamentoSolicitacaoFactory };

use Teknisa\Model\Gestaopessoas\Entities\{ GpeFaseaprovsoli, GpeFasetramsoli, GpeSolicitacao };

class SolicitacaoService extends Service {

    /** @var EstruturaService */
    protected $estruturaService;

    /** @var EmailService */
    protected $emailService;

    /** @var Environment */
    protected $environment;

    /** @var SolicitacaoRepository */
    protected $solicitacaoRepository;

    /** @var array */
    protected $solicitacaoInfo;

    /** @var EntityManager */
    protected $entityManager;
    
    const IDAPROVACAOSEC_APROVADO       = 'A';
    const IDAPROVACAOSEC_PENDENTE       = 'P';
    const IDAPROVACAOSEC_REPROVADO      = 'R';
    const IDAPROVACAOSEC_EMANDAMENTO    = 'E';

    public function __construct(
        EntityManager            $entityManager,
        SolicitacaoRepository    $solicitacaoRepository,
        EscalaRepository         $escalaRepository,
        OcupacaoRepository       $ocupacaoRepository,
        MovimentacaoRepository   $movimentacaoRepository,
        ValeTransporteRepository $valeTransporteRepository,
        FeriasRepository         $feriasRepository,
        AlteSalarioRepository    $alteSalarioRepository,
        MinhaHistoriaRepository  $minhaHistoriaRepository,
        VinculoRepository        $vinculoRepository,
        PeriodoRepository        $periodoRepository,
        Environment              $environment,
        EstruturaService         $estruturaService,
        Utilities $utilities
    ) {
        $this->entityManager            = $entityManager;
        $this->solicitacaoRepository    = $solicitacaoRepository;
        $this->escalaRepository         = $escalaRepository;
        $this->ocupacaoRepository       = $ocupacaoRepository;
        $this->movimentacaoRepository   = $movimentacaoRepository;
        $this->valeTransporteRepository = $valeTransporteRepository;
        $this->feriasRepository         = $feriasRepository;
        $this->alteSalarioRepository    = $alteSalarioRepository;
        $this->minhaHistoriaRepository  = $minhaHistoriaRepository;
        $this->vinculoRepository        = $vinculoRepository;
        $this->periodoRepository        = $periodoRepository;
        $this->environment              = $environment;
        $this->estruturaService         = $estruturaService;
        $this->utilities                = $utilities;
    }

    private $labelFieldPessoa = [
        "NRVINCULOM" => "Nr. Vínculo", "NMVINCULOM" => "Nome Vínculo", "NMOCUPACAOH" => "Ocupação (Cargo)", "VRSALARIO" => "Valor Salário"
    ];
    
    public function getEntityRepositoryName(): string {
        return Repositories::GPE_SOLICITACAO;
    }

    public function getEntityKeysName(): array {
        return [
            'NRORG',
            'NRSOLICITACAO'
        ];
    }

    /**
     * Obtém as solicitações de vínculos de gestores com base nas estruturas organizacionais e no perfil do operador.
     * 
     * A lógica da função se baseia no perfil do operador e na estrutura fornecida:
     * 1. Se o operador tiver o perfil SESMT ou se o número da estrutura fornecida for -1, 
     *    a função irá retornar todas as solicitações de todas as estruturas da organização.
     * 2. Se a estrutura for do tipo "Legal", a função irá buscar apenas as solicitações cujo o vínculo esteja vinculado 
     *    à estrutura legal especificada.
     * 3. Se a estrutura não for "Legal", a função buscará as solicitações das estruturas hierarquicamente 
     *    abaixo da fornecida. Dependendo da quantidade de estruturas:
     *    - Se o número de estruturas for menor que 1000, será usada a função INSTR para realizar a busca.
     *    - Se o número de estruturas for 1000 ou mais, será utilizada uma tabela temporária com JOIN para otimizar a consulta.
     * 
     * @param int $nrEstrutura Número da estrutura fornecida.
     * @param string|null $cdOperadorAprovante Código do operador aprovante, opcional.
     * @return array Retorna um array com as solicitações de vínculos dos gestores.
    */
    public function getSolicitacoesVinculosGestor(
        int     $nrEstrutura,
        ?string $cdOperadorAprovante = null
    ): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $dtmescompetenc = DateUtil::getPrimeiroDiaDoMes(new \Setforms2\Shared\ORM\DateTime())->format(DateUtil::FORMATO_BRASILEIRO);

        $perfilSesmt = null;
        $nrEstruturasString = null;
        $nrEstruturaLegal = null;
        $nrEstruturas = [];

        $perfilSesmt = $this->solicitacaoRepository->verificaPerfilAcessoSST($nrOrg, $cdOperadorAprovante);

        if(!empty($perfilSesmt)) {
            $perfilSesmt = true;
        } else {
            if ($nrEstrutura !== -1) {

                $estrutura = Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findOneBy(array("nrorg"=> $nrOrg,"nrestruturam" => $nrEstrutura));
                $paramOrg = Utils::getEntityManager()->getRepository(Repositories::GPE_PARAMORG)->findOneBy(['nrorg' => $nrOrg]);
                $isTipoLegal = $estrutura->getNrtipoestrutura() == $paramOrg->getNrtpestrutlegal();

                if($isTipoLegal) {
                    // Se a estrutura for do tipo "Legal", armazena o número da estrutura legal.
                    $nrEstruturaLegal = $nrEstrutura;
                } else {
                    // Caso contrário, obtém todas as estruturas hierarquicamente abaixo da fornecida.
                    $nrEstruturas = $this->estruturaService->getTodasEstruturasInferiores($nrEstrutura, true);
    
                    $nrEstruturasString = ArrayUtil::toStringSQL($nrEstruturas);
                }
            }
        }

        $isGetAllSolicitacoes = empty($nrEstruturasString);

        $params = [
            'P_NRORG'                   => $nrOrg,
            'P_DTMESCOMPETENC'          => $dtmescompetenc,
            'P_NRESTRUTURAS'            => $nrEstruturasString,
            'P_TIPOLEGAL'               => $nrEstruturaLegal,
            'P_CDOPERADOR_APROVANTE'    => $cdOperadorAprovante,
            'P_NRPERFAPROVASOLI'        => $perfilSesmt,
        ];

        $nrEstruturasLength = count($nrEstruturas);

        if($isGetAllSolicitacoes || $nrEstruturasLength < 1000) {
            $solicitacoes = $this->solicitacaoRepository->getSolicitacoesVinculosGestor($params);
        } else {
            unset($params['P_NRESTRUTURAS'], $params['P_TIPOLEGAL']);
            $solicitacoes = $this->solicitacaoRepository->getSolicitacoesVinculosGestorByTempLista($params, $nrEstruturas);
        }

        foreach ($solicitacoes as &$solicitacao) {
            $dsobssolicitacao = $this->abreviarObservacaoSolicitacao($solicitacao['DSOBSSOLICITACAO']);
            $solicitacao['DSOBSSOLICITACAOABREV'] = $dsobssolicitacao;
        }      

        return $solicitacoes;
    }

    public function abreviarObservacaoSolicitacao($dsobssolicitacao) {
        try {
            // Padrão para encontrar o trecho LGPD a ser removido
            $padrao = '/\(Usuário concordou com aceite de privacidade \(LGPD\) em \d{2}\/\d{2}\/\d{4} às \d{2}:\d{2}:\d{2}\)/';

            // Substitui o trecho encontrado pelo regex por uma string vazia
            $dsobssolicitacaoNotLGPD = preg_replace($padrao, '', $dsobssolicitacao);

            $maxCaracteres = 80;
            // Remover quebras de linha e caracteres em branco
            $dsobssolicitacao = preg_replace('/[\n\r]+|\s{2,}/', ' ', $dsobssolicitacaoNotLGPD);

            if (mb_strlen($dsobssolicitacao, 'UTF-8') > $maxCaracteres) {
                return mb_substr($dsobssolicitacao, 0, $maxCaracteres, 'UTF-8') . '...';
            } else {
                return $dsobssolicitacao;
            }
        } catch (\Exception $e) {
            return $dsobssolicitacao;
        }
    }

    public function getItemSolicitacao($nrsolicitacao) {
        $nrorg = $this->environment->getNrorg();
        return $this->solicitacaoRepository->getItemSolicitacao($nrorg, $nrsolicitacao);
    }

    public function getSolicitacao($nrsolicitacao) {
        $nrorg = $this->environment->getNrorg();
        return $this->solicitacaoRepository->getSolicitacao($nrorg, $nrsolicitacao);
    }

    public function getTiposolicitacao(?int $nrTipoSolicitacao = null, ?int $nrVinculom = null) {
        $nrorg = $this->environment->getNrorg();
        $tipos = $this->solicitacaoRepository->getTiposolicitacao($nrorg, $nrTipoSolicitacao);

        foreach($tipos as &$tipo) {
            if ($tipo['NRTIPOSOLICITACAO'] == '6') {
                $tipo['NMTIPOSOLICITACAO'] = 'Solicitação de Seleção de Pessoal';
            }

            /* Preenchimento das informacoes do relacionamento do vinculo com o modelo de aprovacao */

            if (!empty($nrVinculom)) {
                $relacModeloTipoAprovSoli = $this->entityManager->getRepository(Repositories::GPE_SOLICITACAO)->getRelacionamentoModeloTipoSoli(
                    $nrorg, $tipo['NRTIPOSOLICITACAO'], $nrVinculom, Utils::getParamters()->getDtmescomptencDiaCorrente()
                );

                if (empty($relacModeloTipoAprovSoli)) {
                    continue;
                }

                $tipo['NRPERFACESTIPOSOLI'] = $relacModeloTipoAprovSoli['NRPERFACESTIPOSOLI'] ?? null;
                $tipo['NRMODELOAPROVSOLI'] = $relacModeloTipoAprovSoli['NRMODELOAPROVSOLI'] ?? null;
                $tipo['DSCONSIDERACAO'] = $relacModeloTipoAprovSoli['DSCONSIDERACAO'] ?? null;
            }
        }

        return $tipos;
    }

    public function getTipoRelacionamento() {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $tiposRel = $this->solicitacaoRepository->getTipoRelacionamento(
            $nrOrg, $nrOrgPadrao
        );

        return $tiposRel;
    }

    public function getLinhaTransporteAtiva(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = date('1/m/Y');

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_DTMESCOMPETENC'  => $dtMesCompetencia,
        ];

        $linhas = $this->solicitacaoRepository->getLinhaTransporteAtiva(
            $params
        );

        return $linhas;
    }

    public function getTipolinhavinc(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $tiposLinhas = $this->solicitacaoRepository->getTipolinhavinc(
            $nrOrg, $nrOrgPadrao
        );

        return $tiposLinhas;
    }

    public function getIntegtransp(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = date('01/m/Y');

        $params = [
            'P_NRORG'               => $nrOrg,
            'P_NRORG_PADRAO'        => $nrOrgPadrao,
            'P_DTMESCOMPETENC'      => $dtMesCompetencia,
        ];

        $numerosIntegracao = $this->solicitacaoRepository->getIntegtransp(
            $params
        );

        return $numerosIntegracao;
    }

    public function getSentidolinha(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $numerosIntegracao = $this->solicitacaoRepository->getSentidolinha(
            $nrOrg, $nrOrgPadrao
        );

        return $numerosIntegracao;
    }

    public function getPessoaRelacionamentos(
        int $nrPessoa
    ): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = date('01/m/Y');

        $params = [
            'P_NRORG'               => $nrOrg,
            'P_NRORGPADRAO'         => $nrOrgPadrao,
            'P_DTMESCOMPETENC'      => $dtMesCompetencia,
            'P_NRPESSOA'            => $nrPessoa,
            'P_NRTIPORELACIONA'     => null,
        ];

        $pessoas = $this->solicitacaoRepository->getPessoaRelacionamentos(
            $params
        );

        return $pessoas;
    }

    public function getTipoDependente(): array {
        $nrOrgPadrao = $this->environment->getNrOrgPadrao();

        $tiposDep = $this->solicitacaoRepository->getTipoDependente($nrOrgPadrao);

        return $tiposDep;
    }

    public function getTemplate(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $templates = $this->solicitacaoRepository->getTemplate($nrOrg);

        return $templates;
    }

    public function getTipoprocessos(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $tipos = $this->solicitacaoRepository->getTipoprocessos($nrOrg);

        return $tipos;
    }

    public function getParcnegocioVinculos(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $tipos = $this->solicitacaoRepository->getParcnegocioVinculos($nrOrg);

        return $tipos;
    }

    public function getTipoTransferencia(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $tipos = $this->solicitacaoRepository->getTipoTransferencia(
            $nrOrg, $nrOrgPadrao
        );

        return $tipos;
    }

    public function getMovimentacaoTransferencia(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = date('01/m/Y');

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRORG_PADRAO' => $nrOrgPadrao,
            'P_DTMESCOMPETENC' => $dtMesCompetencia,
        ];

        $tipos = $this->solicitacaoRepository->getMovimentacaoTransferencia(
            $params
        );

        return $tipos;
    }

    public function getMotivoTransferencia(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $motivos = $this->solicitacaoRepository->getMotivoTransferencia(
            $nrOrg
        );

        return $motivos;
    }

    public function getFasesSolicitacao(
        string $nrSolicitacao
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $fases = $this->solicitacaoRepository->getFasesSolicitacao(
            $nrOrg, $nrSolicitacao
        );

        return $fases;
    }
    
    public function aprovaSolicitacao(
        string  $nrSolicitacao,
        string  $cdOperadorAprovante,
        ?string $dsObservacao           = null,
        ?array  $extraRow               = [],
        ?bool   $isVariasSolicitacoes   = false
    ): array {
        return $this->atualizaSolicitacao(
            $nrSolicitacao,
            self::IDAPROVACAOSEC_APROVADO,
            $cdOperadorAprovante,
            $dsObservacao,
            $extraRow,
            $isVariasSolicitacoes
        );
    }
    
    public function reprovaSolicitacao(
        string  $nrSolicitacao,
        string  $cdOperadorAprovante,
        ?string $dsObservacao           = null,
        ?array  $extraRow               = []
    ): array {
        return $this->atualizaSolicitacao(
            $nrSolicitacao,
            self::IDAPROVACAOSEC_REPROVADO,
            $cdOperadorAprovante,
            $dsObservacao,
            $extraRow
        );
    }
    
    /**
     * Atualiza o status e as fases de aprovacao de solicitacao. Caso a ultima fase for aprovada procede com as insercoes de acordo com as entidades ProcessmentoSolicitacao.
     * 
     * @param string  $nrSolicitacao
     * @param string  $idAprovacao
     * @param string  $cdOperadorAprovante
     * @param ?string $dsObservacao
     * @param ?array  $extraRow
     * 
     * @return bool
     */
    public function atualizaSolicitacao(
        string  $nrSolicitacao,
        string  $idAprovacao,
        string  $cdOperadorAprovante,
        ?string $dsObservacao           = null,
        ?array  $extraRow               = [],
        ?bool   $isVariasSolicitacoes   = false
    ): array {
        $solicitacao = $this->entityManager->getRepository(Repositories::GPE_SOLICITACAO)->findOneBy(array('nrorg' => $this->environment->getNrOrg(), 'nrsolicitacao' => $nrSolicitacao));
        if (empty($solicitacao)) throw new \Exception('Solicitação não encontrada.');
        
        $this->setSolicitacaoInfo($solicitacao->getNrsolicitacao(), $isVariasSolicitacoes);

        /* Chamando a próxima fase no início, pois será usada em vários lugares, evitando repetições de consulta ao banco */

        $nrFaseSeguinte = $this->solicitacaoInfo['NRFASEAPROVSOLI_SEGUINTE'];
        $proximafase = $this->entityManager->getRepository(Repositories::GPE_FASEAPROVSOLI)->findOneBy(['nrorg' => $this->environment->getNrorg(), 'nrfaseaprovsoli' => $nrFaseSeguinte]);
        
        /* Checa se a solicitacao esta em um estado que permite a aprovacao */

        if (!$this->isSolicitacaoAtualValidaAprovacao()) {
            throw new ExpectedException('Não foi possível aprovar. A solicitação já está aprovada/rejeitada');
        }

        /* Checa se o operador que esta realizando a aprovacao tem acesso */

        if (!$this->checkPerfilAprovacaoSolicitacaoAtual($solicitacao, $proximafase, $cdOperadorAprovante)) {
            throw new ExpectedException('Operador não possui perfil de acesso para aprovar essa fase da solicitação. Reveja as informações do modelo de fases da aprovação solicitação.');
        }

        /* Invoca as rotinas de aprovacao se a solicitacao esta na ultima fase */

        if ($this->isSolicitacaoAtualEmUltimaEtapa() && $idAprovacao === self::IDAPROVACAOSEC_APROVADO) {
            $processamentoSolicitacao = $this->getProcessamentoSolicitacao($solicitacao, $extraRow);
            if (!empty($processamentoSolicitacao)) {
                $processamentoSolicitacao->efetivar();
            }
        }

        if($idAprovacao === self::IDAPROVACAOSEC_REPROVADO) {
            $processamentoSolicitacao = $this->getProcessamentoSolicitacao($solicitacao, $extraRow);
            $processamentoSolicitacao->rejeitar();
        }

        /* Atualiza a informacoes da solicitacao e de tramitacao */

        $idaprovacao = $this->aprovaFaseTramitacao($solicitacao->getNrsolicitacao(), $solicitacao->getNrtiposolicitacao(), $proximafase, $idAprovacao, $cdOperadorAprovante, $dsObservacao);

        $this->update(
            [
                'NRORG'             => $this->environment->getNrorg(),
                'NRSOLICITACAO'     => $nrSolicitacao,
                'IDAPROVACAOSEC'    => $idaprovacao,
                'CDOPERULTATU'      => $cdOperadorAprovante,
            ]
        );
        
        return [
               'SUCCESS'        => true,
               'IDAPROVACAOSEC' => $idaprovacao,
            ];
    }
    
    /**
     * Valida a existencia de um modelo para um determinado tipo de solicitacao.
     * 
     * @param $nrtiposolicitacao
     * 
     * @return array
     */
    public function validaModeloAprovacao($nrtiposolicitacao) {
        $erro["Bloquear"] = false;
        
        $perfil = $this->validaPerfildeAcesso($nrtiposolicitacao);
        if(!$perfil) {
            $erro["Texto"] = "Não existe perfil de acesso para este tipo de solicitação.";
            $erro["Bloquear"] = true;
        } else {
            if(!$perfil->getNrmodeloaprovsoli()) {
                $erro["Texto"] = "Modelo de aprovação não parametrizado para este tipo de solicitação.";
                $erro["Bloquear"] = true;
            }
        }

        return $erro;
    }
    
    /**
     * Busca o registro de GPE_PERFACESTIPOSOLI associado ao tipo de solicitacao.
     * 
     * @param $tiposoli
     * 
     * @return ?GpePerfacestiposoli
     */
    public function validaPerfildeAcesso($tiposoli){
        $perfil = $this->entityManager->getRepository(Repositories::GPE_PERFACESTIPOSOLI)->findOneBy(array('nrorg'=>$this->environment->getNrorg(),'nrtiposolicitacao'=>$tiposoli));
        if (!$perfil) {
            return false;
        }
        return $perfil;
    }
    
    /**
     * Retorna o proximo IDAPROVACAOSEC a ser atualizado na solicitacao. Caso nao existam proximas fases, null eh retornado.
     * 
     * @param string  $nrSolicitacao
     * @param string  $nrTipoSolicitacao
     * @param string  $idAprovacao
     * @param string  $cdOperadorAprovante
     * @param ?string $dsObservacao
     * 
     * @return ?string
     */
    public function aprovaFaseTramitacao(
        string  $nrSolicitacao,
        string  $nrTipoSolicitacao,
        $proximafase,
        string  $idAprovacao,
        string  $cdOperadorAprovante,
        ?string $dsObservacao = null
    ): ?string {

        $faseatual = $this->entityManager->getRepository(Repositories::GPE_FASETRAMSOLI)->retornaFaseatual($this->environment->getNrorg(), $nrSolicitacao, Repositories::class);

        if ($faseatual) {
            $nrordem = $faseatual->getNrordem();
        } else {
            $nrordem = 0;
        }

        $naoExisteProximaFase = empty($proximafase);
        if ($naoExisteProximaFase) {
            return null;
        }

        if ($idAprovacao == 'A') {
            $idaprovado = 'S';
        } else {
            $idaprovado = 'N';
        }

        $fasetram = $this->insereFaseTramSoli($nrSolicitacao, $proximafase->getNrfaseaprovsoli(), $idaprovado, $cdOperadorAprovante, $dsObservacao);

        $ultimafase = $this->entityManager->getRepository(Repositories::GPE_FASEAPROVSOLI)->findOneBy(
            array('nrorg' => $this->environment->getNrorg(), 'nrmodeloaprovsoli' => $proximafase->getNrmodeloaprovsoli()),
            array('nrordem' => 'DESC')
        );

        /*
        Se for aprovar e a fase for a última do modelo, muda o status da solicitação para aprovada
        Se for aprovar e não for a última fase o status da solicitação fica como em andamento
        Se for reprovar e a fase for obrigatória muda o status da solicitação para reprovada
        Se for reprovar e a fase não for obrigatória o status da solicitação fica como em andamento
        */
        $novoIdAprovacao = null;
        
        if ($idAprovacao == 'A') {
            if ($proximafase->getNrfaseaprovsoli() == $ultimafase->getNrfaseaprovsoli()) {
                $novoIdAprovacao = 'A';
            } else {
                $novoIdAprovacao = 'E';
            }
        } else {
            if ($proximafase->getIdfaseobrigatoria() == 'S') {
                $novoIdAprovacao = 'R';
            } else {
                $novoIdAprovacao = 'E';
            }
        }

        return $novoIdAprovacao;
    }

    /**
     * Verifica se o operador aprovante possui perfil de acesso adequado para a fase de aprovacao da solicitacao
     * @param string $nrSolicitacao
     * @param string $cdOperadorAprovante
     * @return bool
     */
    public function checkPerfilAprovacaoSolicitacaoAtual(
        $solicitacao,
        $proximafase,
        string $cdOperadorAprovante
    ): bool {

        if (empty($solicitacao)) {
            return false;
        }

        $naoExisteProximaFase = empty($proximafase);
        if ($naoExisteProximaFase) {
            return null;
        }
        
        $perfilfase = explode(',',$proximafase->getDsnrperfilacesso());

        $operador = $this->entityManager->getRepository(Repositories::OPERADOR)->findOneBy(['cdoperador' => $cdOperadorAprovante]);

        $perfiloper = $this->entityManager->getRepository(Repositories::PERFACESSPARC)->findBy(
            ['nrorg' => $this->environment->getNrorg(), 'nrparcnegocio' => $operador->getNrparcnegocio(), 'nrperfilacesso' => $perfilfase]
        );

        return !empty($perfiloper);
    }
    
    /**
     * Inclui o registro de uma fase para uma solicitacao.
     * 
     * @param string  $nrSolicitacao,
     * @param string  $nrFaseAprovSoli,
     * @param string  $idAprovado,
     * @param string  $cdOperadorAprovante,
     * @param ?string $dsObservacao = null
     * 
     * @return GpeFasetramsoli
     */
    public function insereFaseTramSoli(
        string  $nrSolicitacao,
        string  $nrFaseAprovSoli,
        string  $idAprovado,
        string  $cdOperadorAprovante,
        ?string $dsObservacao = null
    ): GpeFasetramsoli {

        /** @var \Teknisa\Model\Gestaopessoas\Entities\GpeFasetramsoli $gpe_fasetramsoli */
        $gpe_fasetramsoli = Repositories::GPE_FASETRAMSOLI;    
        
        $faseTram = new $gpe_fasetramsoli();

        $faseTram->setNrsolicitacao($nrSolicitacao);
        $faseTram->setNrfaseaprovsoli($nrFaseAprovSoli);
        $faseTram->setIdaprovado($idAprovado);
        $faseTram->setDsobservacao($dsObservacao);
        $faseTram->setNrorg($this->environment->getNrorg());
        $faseTram->setDtinclusao(DateUtil::getDataAtual());
        $faseTram->setCdoperinclusao($cdOperadorAprovante);
        $faseTram->setNrorginclusao($this->environment->getNrorg());
        
        $nrFaseTramSoli = NovoCodigo::geraCodigo($this->environment->getNrorg(), 'GPE_FASETRAMSOLI', 1, 12, true);
        $faseTram->setNrfasetramsoli($nrFaseTramSoli);
        
        $this->entityManager->persist($faseTram);
        $this->entityManager->flush();
        
        return $faseTram;
    }
    
    /**
     * Busca a entidade de processamento da aprovacao de uma solicitacao capazes de efetiva-las.
     * 
     * @param GpeSolicitacao  $solicitacao,
     * @param ?array          $extraRow
     * 
     * @return ?ProcessamentoSolicitacao
     */
    public function getProcessamentoSolicitacao(
        GpeSolicitacao  $solicitacao,
        ?array          $extraRow
    ): ?ProcessamentoSolicitacao {
        return ProcessamentoSolicitacaoFactory::createProcessamentoSolicitacao($solicitacao, $extraRow);
    }
    
    public static function isTipoSoliOnlyEmail(string $tipoSoli): bool {
        return $tipoSoli < 1;
    }

    /**
     * Verifica se a solicitacao atual esta em sua ultima etapa
     * @return bool
     */
    public function isSolicitacaoAtualEmUltimaEtapa(): bool {
        return !empty($this->solicitacaoInfo)
            ? $this->solicitacaoInfo['FASE_ATUAL_EH_ULTIMA'] == 'S'
            : false;
    }

    /**
     * Verifica se a solicitacao atual a ser analisada eh valida ou nao
     * @return bool
     */
    public function isSolicitacaoAtualValidaAprovacao(): bool {
        return !empty($this->solicitacaoInfo)
            ? $this->solicitacaoInfo['FASE_ATUAL_VALIDA_APROVACAO'] == 'S'
            : false;
    }

    public function getInfoFase(int $nrSolicitacao): array {
        $nrOrg = $this->environment->getNrorg();
        $infoFase = [];

        if($nrOrg == 3260 || $nrOrg == 1410) {
            $infoFase = $this->solicitacaoRepository->getInfoFase($nrOrg, $nrSolicitacao);
        } else {
            $infoFase = $this->entityManager->getRepository(Repositories::GPE_SOLICITACAO)->getInfoFase(
                $nrOrg, $nrSolicitacao
            );
        }

        return empty($infoFase) ? [] : $infoFase;
    }

    public function getInfoFaseWithEmails(int $nrSolicitacao): array {
        $nrOrg = $this->environment->getNrorg();
        $infoFase = [];

        $infoFase = $this->solicitacaoRepository->getInfoFaseWithEmails($nrOrg, $nrSolicitacao);

        return empty($infoFase) ? [] : $infoFase;
    }

    /**
     * Responsavel por definir as informacoes e status das ocorrencias
     * 
     * @param int $nrSolicitacao
     * @return bool
     */
    private function setSolicitacaoInfo(
        int $nrSolicitacao,
        bool $isVariasSolicitacoes = false
    ): void {
        if($isVariasSolicitacoes) {
            $this->solicitacaoInfo = array();
        }
        if (!empty($this->solicitacaoInfo)) {
            return;
        }

        $nrOrg = (int) $this->environment->getNrOrg();

        if($nrOrg == 3260 || $nrOrg == 1410) {

            $this->solicitacaoInfo = $this->solicitacaoRepository->getInfoFase($nrOrg, $nrSolicitacao);
            
            return;
        }

        $this->solicitacaoInfo = $this->entityManager->getRepository(Repositories::GPE_SOLICITACAO)->getInfoFase(
            $nrOrg, $nrSolicitacao
        );
    }
    
    public function getAlteracaoEscalaTrabalho(
        int $nrVinculo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $escalaTrabalho = $this->escalaRepository->getEscalaTrabalhoVinculo($nrOrg, $nrVinculo);
        $nrEscalaTraBM = $escalaTrabalho["NRESCALATRABM"] ?? 0;

        $turno = $this->escalaRepository->getTurnoVinculo($nrOrg, $nrVinculo, $nrEscalaTraBM);
    
        $result = [
            'ESCALATRABALHO' => $escalaTrabalho,
            'TURNO' => $turno
        ];

        return $result;
    }

    public function getAlteracaoOcupacao(
        int $nrVinculo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $ocupacao = $this->ocupacaoRepository->getOcupacaoVinculo($nrOrg, $nrVinculo);

        return $ocupacao;
    }

    public function getAlteracaoTransferencia(
        int $nrVinculo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $legal = $this->movimentacaoRepository->getUltimaMovimentacaoLegal($nrOrg, $nrVinculo);
        $Geren = $this->movimentacaoRepository->getUltimaMovimentacaoGeren($nrOrg, $nrVinculo);

        $result = $this->getAlteracaoEscalaTrabalho($nrVinculo);

        $result = [
            'LEGAL' => $legal,
            'GERENCIAL' => $Geren
        ];

        return $result;
    }

    public function getLinhasVinculosVT(
        int $nrVinculo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $dtAtual = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO);

        $linhas = $this->valeTransporteRepository->getLinhasVinculoVT($nrOrg, $nrVinculo, $dtAtual);

        $tarifas = $this->valeTransporteRepository->getTarifasVT($dtAtual);

            
        $result = [
            'LINHAS' => $linhas,
            'TARIFAS' => $tarifas
        ];

        return $result;
    }

    public function getDadosFeriasVinculo(
        int $nrVinculo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $programacaoFerias = $this->feriasRepository->getFeriasVinculoValida($nrOrg, $nrVinculo);

        return $programacaoFerias;
    }

    public function getSolicitacaoSubstituicaoOrVinculoSubst(
        int $nrVinculo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrTipoSolicitacao = ProcessamentoSolicitacaoFactory::TP_SOLI_SUBSTITUICAO;

        $solicitacoes = $this->solicitacaoRepository->getSolicitacaoByTipoEVinculo($nrOrg, $nrVinculo, $nrTipoSolicitacao);
        $vinculo = $this->solicitacaoRepository->getVinculoSubstituido($nrOrg, $nrVinculo);

        $result = [
            'SOLICITACOES' => $solicitacoes,
            'VINCULO' => $vinculo
        ];

        return $result;
    }

    public function setDataSoliSubstituicaoVinculo(
        int $nrVinculo,
        int $nrSolicitacao
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();
        $dtmescompetenc = $this->environment->getDtmescomptencDiaCorrente();
        $cdoperador = Utils::getParamters()->getCdoperador();

        $ocupacao = $this->ocupacaoRepository->getOcupacaoVinculo($nrOrg, $dtmescompetenc, $nrVinculo);
        $ultimoSalario = $this->alteSalarioRepository->getUltimoSalario($nrOrg, $dtmescompetenc, $nrVinculo);
        $vinculo = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array('nrorg' => $nrOrg, 'nrvinculom' => $nrVinculo));

        $itens = [
            "NRVINCULOM"  => $nrVinculo,
            "NMVINCULOM"  => $vinculo->getNmvinculom(),
            "NMOCUPACAOH" => $ocupacao[0]["NMOCUPACAOH"],
            "VRSALARIO"   => $ultimoSalario[0]["VRSALARIO"]
        ];

        foreach ($itens as $key => $field) {
            if(!empty($field)){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrOrg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrSolicitacao, $key, $this->labelFieldPessoa[$key], strval($field), $nrOrg, $cdoperador);
            }
        }

    }

    public function getFolhaFechada(
        int $nrVinculo,
        string $dtAlteracao
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $folha = $this->solicitacaoRepository->getFolhaFechada($nrOrg, $nrVinculo, $dtAlteracao);

        return $folha;
    }

    public function getHierarquiaByEstruturaLegalAndGeren(
        int $nrEstrutLegal,
        int $nrEstrutGeren
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $dtAtual = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO);

        $hierarquia = $this->solicitacaoRepository->getHierarquiaByEstruturaLegalAndGeren($nrOrg, $nrEstrutLegal, $nrEstrutGeren, $dtAtual);

        return $hierarquia;
    }

    public function getDadosVinculo(
        int $nrVinculo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $vinculoh = $this->vinculoRepository->getVinculoHistorico($nrOrg, $nrVinculo);

        return $vinculoh;
    }

    public function getSolicitacoesAdmissao(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $solicitacoes = $this->solicitacaoRepository->getSolicitacoesAdmissao($nrOrg);

        return $solicitacoes;
    }
    
    public function resetFases($nrsolicitacao, $nrorg) {
        try {
            $params = array(
                "P_NRSOLICITACAO"      => $nrsolicitacao,
                "P_NRORG"              => $nrorg
            );

            $this->solicitacaoRepository->resetFases($params);
            
            $this->solicitacaoRepository->resetStatus($params);
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function updateAnexo($row) {
        try {
            if (isset($row['DSFILE']) && isset($row['NMFILE'])) {
                $fileContent    = $row['DSFILE'];
                $fileName       = $row['NMFILE'];

                $row['DSPATH'] = $this->saveFile($row);

                unset($row['DSFILE']);
                unset($row['NMFILE']);
            }
            
            $params = array(
                "P_NRSOLICITACAO"      => $row['NRSOLICITACAO'],
                "P_NRORG"              => $row['NRORG'],
                "P_DSPATH"             => isset($row['DSPATH']) ? $row['DSPATH'] : null,
            );

            $this->solicitacaoRepository->updateAnexo($params);            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getMotivoRecusa($nrsolicitacao) {
        try {
            $nrorg = $this->environment->getNrorg();
            
            $params = array(
                "P_NRSOLICITACAO"      => $nrsolicitacao,
                "P_NRORG"              => $nrorg
            );

            $motivo = $this->solicitacaoRepository->getMotivoRecusa($params);
            
            return $motivo;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveFile($row) {
        $nrOrg = (int) $this->environment->getNrOrg();

        if (isset($row['DSFILE']) && $row['DSFILE'] !== '') {
            $file = $row['DSFILE'];

            $filePath = $this->utilities->saveFileOnFileServer(
                $file, 'jpeg', $nrOrg, null, null, '', false
            );

            return $filePath;
        }

        return NULL;
    }

    public function getTodosVinculosRescindidos(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $periodoParams = [
            'P_NRORG'          => $nrOrg,
            'P_NRTIPOVINCULOM' => 1
        ];

        $periodo = $this->periodoRepository->getPeriodoAtual($periodoParams);

        $dtMesCompetencia = Utilities::formatDate($periodo['DTFINALAPURACAO']);

        $params = [
            'P_NRORG'             => $nrOrg,
            'P_DTMESCOMPETENC'    => $dtMesCompetencia,
        ];

        $vinculos = $this->vinculoRepository->getTodosVinculosRescindidos($params);

        return $vinculos;
    }
    public function setDocumentoSolicitacao($row) {
        $nrOrg = (int) $this->environment->getNrOrg();
        $cdOperador = (string) $this->environment->getCdOperador();
        $params = [];

        $params['NRORG']                        = $row['NRORG'] ?? $nrOrg;
        $params['NRSOLICITACAODOCUMENTO']       = NovoCodigo::geraCodigo($nrOrg, 'GPE_SOLICITACAODOCUMENTO', 1, 12, true, false);
        $params['NRSOLICITACAO']                = (int) $row['NRSOLICITACAO'];
        $params['DSPATH']                       = $row['DSPATH'] ?? null;
        $params['DSOBSSOLICITACAODOCUMENTO']    = $row['DSOBSSOLICITACAODOCUMENTO'] ?? null;
        $params['NMFILE']                       = $row['NMFILE'] ?? null;
        $params['IDTYPE']                       = $row['IDTYPE'] ?? null;
        $params['IDATIVO']                      = $row['IDATIVO'] ?? 'S';
        $params['NRORGINCLUSAO']                = $row['NRORGINCLUSAO'] ?? $nrOrg;
        $params['DTINCLUSAO']                   = $row['DTINCLUSAO'] ?? date('d/m/Y');
        $params['CDOPERINCLUSAO']               = $row['CDOPERINCLUSAO'] ?? $cdOperador;
        $params['NRORGULTATU']                  = $row['NRORGULTATU'] ?? null;
        $params['DTULTATU']                     = $row['DTULTATU'] ?? null;
        $params['CDOPERULTATU']                 = $row['CDOPERULTATU'] ?? null;

        $this->solicitacaoRepository->setDocumentoSolicitacao($params);

        return $params['NRSOLICITACAODOCUMENTO'];
    }

    public function getDadosSolicitacao(
        int $nrSolicitacao
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $dados = $this->solicitacaoRepository->getDadosSolicitacao($nrOrg, $nrSolicitacao);
            
        return $dados;
    }

    public function getDadosSolicitacaoToExport($filters, $reportName, $nrorg) {
        $tipoLegal = null;
        
        $params = array(
            'P_NRORG' => (int)$nrorg,
            'P_NRESTRUTURAS' => null,
        );

        if(isset($filters['NRESTRUTLEGAL'])) {
            $params['P_TIPOLEGAL'] = $filters['NRESTRUTLEGAL'];
            unset($filters['NRESTRUTLEGAL']);
        }

        if(isset($filters['NRESTRUTGEREN'])) {
            $params['P_NRESTRUTURAS'] = $filters['NRESTRUTGEREN'];
            unset($filters['NRESTRUTGEREN']);
        }

        $itemsQuery = $this->getItemsQuery($reportName, $filters);
        
        foreach($filters as $key => $filter) {
            $params['P_'.$key] = $filter;
        }

        if(!isset($params['P_DSAPROVACAOSEC'])) {
            $params['P_DSAPROVACAOSEC'] = null;
        }

        return $this->solicitacaoRepository->getDadosSolicitacaoComItens($params, $itemsQuery);
    }

    public function getItemsQuery($reportName, &$filters) {
        $itemsQuery = '';
        switch ($reportName){
            case 'horas_extras':
                if(isset($filters['DTAPROVAHE'])) {
                    if(empty($filters['DTAPROVAHE'])) {
                       unset($filters['DTAPROVAHE']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTAPROVAHE');
                    }
                }

                if(isset($filters['QTINIHOR'])) {
                    if(empty($filters['QTINIHOR'])) {
                       unset($filters['QTINIHOR']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('QTINIHOR');
                    }
                }

                if(isset($filters['QTFIMHOR'])) {
                    if(empty($filters['QTFIMHOR'])) {
                       unset($filters['QTFIMHOR']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('QTFIMHOR');
                    }
                }

                
                $filters['NRTIPOSOLICITACAO'] = 18;
                
                return $itemsQuery;
                break;
            case 'alteracao_escala':
                if(isset($filters['DTINIESCALA'])) {
                    if(empty($filters['DTINIESCALA'])) {
                       unset($filters['DTINIESCALA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTINIESCALA');
                    }
                }

                if(isset($filters['NRESCALATRABM'])) {
                    if(empty($filters['NRESCALATRABM'])) {
                       unset($filters['NRESCALATRABM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRESCALATRABM');
                    }
                }

                if(isset($filters['NRTURNO'])) {
                    if(empty($filters['NRTURNO'])) {
                       unset($filters['NRTURNO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTURNO');
                    }
                }

                
                $filters['NRTIPOSOLICITACAO'] = 14;

                return $itemsQuery;
                break;
            case 'alteracao_salarial':
                if(isset($filters['VRSALARIO'])) {
                    if(empty($filters['VRSALARIO'])) {
                       unset($filters['VRSALARIO']); 
                    } else {
                        $filters['VRSALARIO'] = str_replace(",", ".", $filters['VRSALARIO']);
                        $itemsQuery .= $this->getExistsQuery('VRSALARIO');
                    }
                }

                if(isset($filters['IDTPSALARIO'])) {
                    if(empty($filters['IDTPSALARIO'])) {
                       unset($filters['IDTPSALARIO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexAfter('IDTPSALARIO');
                    }
                }

                if(isset($filters['DTALTESALARIO'])) {
                    if(empty($filters['DTALTESALARIO'])) {
                       unset($filters['DTALTESALARIO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTALTESALARIO');
                    }
                }

                
                $filters['NRTIPOSOLICITACAO'] = 3;

                return $itemsQuery;
                break;
            case 'alteracao_ocupacao':
                if(isset($filters['NROCUPACAOM'])) {
                    if(empty($filters['NROCUPACAOM'])) {
                       unset($filters['NROCUPACAOM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NROCUPACAOM');
                    }
                }

                if(isset($filters['NRMOTIVOOCUPA'])) {
                    if(empty($filters['NRMOTIVOOCUPA'])) {
                       unset($filters['NRMOTIVOOCUPA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRMOTIVOOCUPA');
                    }
                }

                if(isset($filters['NRTIPOOCUPACAO'])) {
                    if(empty($filters['NRTIPOOCUPACAO'])) {
                       unset($filters['NRTIPOOCUPACAO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTIPOOCUPACAO');
                    }
                }

                if(isset($filters['DTINIOCUPACAO'])) {
                    if(empty($filters['DTINIOCUPACAO'])) {
                       unset($filters['DTINIOCUPACAO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTINIOCUPACAO');
                    }
                }

                
                $filters['NRTIPOSOLICITACAO'] = 27;

                return $itemsQuery;
                break;
            case 'aumento_quadro':
                if(isset($filters['NRTURNO'])) {
                    unset($filters['NRTURNO']); 
                }

                if(isset($filters['NMPROCESSO'])) {
                    if(empty($filters['NMPROCESSO'])) {
                       unset($filters['NMPROCESSO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('NMPROCESSO');
                    }
                }

                if(isset($filters['NRTEMPLATE'])) {
                    if(empty($filters['NRTEMPLATE'])) {
                       unset($filters['NRTEMPLATE']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTEMPLATE');
                    }
                }

                if(isset($filters['NRTIPOPROCESSO'])) {
                    if(empty($filters['NRTIPOPROCESSO'])) {
                       unset($filters['NRTIPOPROCESSO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTIPOPROCESSO');
                    }
                }

                if(isset($filters['DTSOLICITA'])) {
                    if(empty($filters['DTSOLICITA'])) {
                       unset($filters['DTSOLICITA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTSOLICITA');
                    }
                }

                if(isset($filters['DTLIMSOLICITA'])) {
                    if(empty($filters['DTLIMSOLICITA'])) {
                       unset($filters['DTLIMSOLICITA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTLIMSOLICITA');
                    }
                }

                if(isset($filters['DTPREVISTAADMISSAO'])) {
                    if(empty($filters['DTPREVISTAADMISSAO'])) {
                       unset($filters['DTPREVISTAADMISSAO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTPREVISTAADMISSAO');
                    }
                }

                if(isset($filters['NRVAGAS'])) {
                    if(empty($filters['NRVAGAS'])) {
                       unset($filters['NRVAGAS']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('NRVAGAS');
                    }
                }

                if(isset($filters['NROCUPACAOM'])) {
                    if(empty($filters['NROCUPACAOM'])) {
                       unset($filters['NROCUPACAOM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NROCUPACAOM');
                    }
                }

                if(isset($filters['NRESCALATRABM'])) {
                    if(empty($filters['NRESCALATRABM'])) {
                       unset($filters['NRESCALATRABM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRESCALATRABM');
                    }
                }

                if(isset($filters['VRSALARIO'])) {
                    if(empty($filters['VRSALARIO'])) {
                       unset($filters['VRSALARIO']); 
                    } else {
                        $filters['VRSALARIO'] = str_replace(",", ".", $filters['VRSALARIO']);
                        $itemsQuery .= $this->getExistsQuery('VRSALARIO');
                    }
                }

                if(isset($filters['NRTIPOVINCULOM'])) {
                    if(empty($filters['NRTIPOVINCULOM'])) {
                       unset($filters['NRTIPOVINCULOM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTIPOVINCULOM');
                    }
                }

                
                $filters['NRTIPOSOLICITACAO'] = 28;

                return $itemsQuery;
                break;
            case 'desligamento':
                if(isset($filters['NRTPAVIPRE'])) {
                    if(empty($filters['NRTPAVIPRE'])) {
                       unset($filters['NRTPAVIPRE']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTPAVIPRE');
                    }
                }

                if(isset($filters['NRTPDEMISSAO'])) {
                    if(empty($filters['NRTPDEMISSAO'])) {
                       unset($filters['NRTPDEMISSAO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTPDEMISSAO');
                    }
                }

                if(isset($filters['NRMOTIVORESC'])) {
                    if(empty($filters['NRMOTIVORESC'])) {
                       unset($filters['NRMOTIVORESC']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRMOTIVORESC');
                    }
                }


                if(isset($filters['DTRESCISAOVINC'])) {
                    if(empty($filters['DTRESCISAOVINC'])) {
                       unset($filters['DTRESCISAOVINC']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTRESCISAOVINC');
                    }
                }

                if(isset($filters['DTRESCISAOPROJETADA'])) {
                    if(empty($filters['DTRESCISAOPROJETADA'])) {
                       unset($filters['DTRESCISAOPROJETADA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTRESCISAOPROJETADA');
                    }
                }

                if(isset($filters['CDPOSSUIBENSDAEMPRESA'])) {
                    if(empty($filters['CDPOSSUIBENSDAEMPRESA'])) {
                       unset($filters['CDPOSSUIBENSDAEMPRESA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('CDPOSSUIBENSDAEMPRESA');
                    }
                }

                if(isset($filters['CDREPOSICAOVAGA'])) {
                    if(empty($filters['CDREPOSICAOVAGA'])) {
                       unset($filters['CDREPOSICAOVAGA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('CDREPOSICAOVAGA');
                    }
                }

                if(isset($filters['CDPESSOANAOGRATA'])) {
                    if(empty($filters['CDPESSOANAOGRATA'])) {
                       unset($filters['CDPESSOANAOGRATA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('CDPESSOANAOGRATA');
                    }
                }

                
                $filters['NRTIPOSOLICITACAO'] = 5;
                
                return $itemsQuery;
                break;
            case 'advertencia':
                if(isset($filters['NRSITUFUNCM'])) {
                    if(empty($filters['NRSITUFUNCM'])) {
                        unset($filters['NRSITUFUNCM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRSITUFUNCM');
                    }
                }

                if(isset($filters['DTINISITUFUNC'])) {
                    if(empty($filters['DTINISITUFUNC'])) {
                        unset($filters['DTINISITUFUNC']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTINISITUFUNC');
                    }
                }

                if(isset($filters['NRQTDIASITUFUNC'])) {
                    if(empty($filters['NRQTDIASITUFUNC'])) {
                        unset($filters['NRQTDIASITUFUNC']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('NRQTDIASITUFUNC');
                    }
                }

                if(isset($filters['DTFIMSITUFUNC'])) {
                    if(empty($filters['DTFIMSITUFUNC'])) {
                        unset($filters['DTFIMSITUFUNC']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTFIMSITUFUNC');
                    }
                }

                
                $filters['NRTIPOSOLICITACAO'] = 20;
                
                return $itemsQuery;
                break;
            
            case 'auxilio_moradia':
                if(isset($filters['DTINIVIGENCIA'])) {
                    if(empty($filters['DTINIVIGENCIA'])) {
                        unset($filters['DTINIVIGENCIA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTINIVIGENCIA');
                    }
                }

                if(isset($filters['DTFIMVIGENCIA'])) {
                    if(empty($filters['DTFIMVIGENCIA'])) {
                        unset($filters['DTFIMVIGENCIA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTFIMVIGENCIA');
                    }
                }

                if(isset($filters['VREVENTOFIXO'])) {
                    if(empty($filters['VREVENTOFIXO'])) {
                        unset($filters['VREVENTOFIXO']); 
                    } else {
                        $filters['VREVENTOFIXO'] = str_replace(",", ".", $filters['VREVENTOFIXO']);
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('VREVENTOFIXO');
                    }
                }

                
                $filters['NRTIPOSOLICITACAO'] = 22;
                
                return $itemsQuery;
                break;
            
            case 'beneficios':
                if(isset($filters['CDTIPOALTEVC'])) {
                    if(empty($filters['CDTIPOALTEVC'])) {
                        unset($filters['CDTIPOALTEVC']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('CDTIPOALTEVC');
                    }
                }

                if(isset($filters['NRPESSOA'])) {
                    if(empty($filters['NRPESSOA'])) {
                        unset($filters['NRPESSOA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRPESSOA');
                    }
                }

                if(isset($filters['NRBENEFICIOM'])) {
                    if(empty($filters['NRBENEFICIOM'])) {
                        unset($filters['NRBENEFICIOM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRBENEFICIOM');
                    }
                }

                if(isset($filters['QTBENEFICIO'])) {
                    if(empty($filters['QTBENEFICIO'])) {
                    unset($filters['QTBENEFICIO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('QTBENEFICIO');
                    }
                }

                if(isset($filters['DTINSCBENEFICIARIO'])) {
                    if(empty($filters['DTINSCBENEFICIARIO'])) {
                    unset($filters['DTINSCBENEFICIARIO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTINSCBENEFICIARIO');
                    }
                }

                if(isset($filters['DTCANCBENEFICIARIO'])) {
                    if(empty($filters['DTCANCBENEFICIARIO'])) {
                    unset($filters['DTCANCBENEFICIARIO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTCANCBENEFICIARIO');
                    }
                }

                
                $filters['NRTIPOSOLICITACAO'] = 24;
                
                return $itemsQuery;
                break;
            
            case 'ferias':
                if(isset($filters['QTDIASFERIAS'])) {
                    if(empty($filters['QTDIASFERIAS'])) {
                    unset($filters['QTDIASFERIAS']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('QTDIASFERIAS');
                    }
                }

                if(isset($filters['DTINIGOZOFERIAS'])) {
                    if(empty($filters['DTINIGOZOFERIAS'])) {
                    unset($filters['DTINIGOZOFERIAS']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTINIGOZOFERIAS');
                    }
                }

                if(isset($filters['DTFIMGOZOFERIAS'])) {
                    if(empty($filters['DTFIMGOZOFERIAS'])) {
                    unset($filters['DTFIMGOZOFERIAS']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTFIMGOZOFERIAS');
                    }
                }

                if(isset($filters['IDADIANT13SALAR'])) {
                    if(empty($filters['IDADIANT13SALAR'])) {
                    unset($filters['IDADIANT13SALAR']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('IDADIANT13SALAR');
                    }
                }
                
                $filters['NRTIPOSOLICITACAO'] = 1;
                
                return $itemsQuery;
                break;
            
            case 'inclusao_dependentes':
                if(isset($filters['NRTIPORELACIONA'])) {
                    if(empty($filters['NRTIPORELACIONA'])) {
                    unset($filters['NRTIPORELACIONA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTIPORELACIONA');
                    }
                }

                if(isset($filters['DTNASCIFUNDPARC'])) {
                    if(empty($filters['DTNASCIFUNDPARC'])) {
                    unset($filters['DTNASCIFUNDPARC']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTNASCIFUNDPARC');
                    }
                }

                if(isset($filters['IDSEXOPESSOA'])) {
                    if(empty($filters['IDSEXOPESSOA'])) {
                    unset($filters['IDSEXOPESSOA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('IDSEXOPESSOA');
                    }
                }
                
                $filters['NRTIPOSOLICITACAO'] = 11;
                
                return $itemsQuery;
                break;
            
            case 'reducao_salarial':
                if(isset($filters['DTALTESALARIO'])) {
                    if(empty($filters['DTALTESALARIO'])) {
                       unset($filters['DTALTESALARIO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTALTESALARIO');
                    }
                }

                if(isset($filters['DTINIESCALA'])) {
                    if(empty($filters['DTINIESCALA'])) {
                       unset($filters['DTINIESCALA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTINIESCALA');
                    }
                }

                if(isset($filters['VRSALARIO'])) {
                    if(empty($filters['VRSALARIO'])) {
                       unset($filters['VRSALARIO']); 
                    } else {
                        $filters['VRSALARIO'] = str_replace(",", ".", $filters['VRSALARIO']);
                        $itemsQuery .= $this->getExistsQuery('VRSALARIO');
                    }
                }

                if(isset($filters['IDTPSALARIO'])) {
                    if(empty($filters['IDTPSALARIO'])) {
                       unset($filters['IDTPSALARIO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('IDTPSALARIO');
                    }
                }

                if(isset($filters['NRTIPOALTERA'])) {
                    if(empty($filters['NRTIPOALTERA'])) {
                       unset($filters['NRTIPOALTERA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTIPOALTERA');
                    }
                }

                if(isset($filters['NRESCALATRABM'])) {
                    if(empty($filters['NRESCALATRABM'])) {
                       unset($filters['NRESCALATRABM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRESCALATRABM');
                    }
                }

                if(isset($filters['NRTURNO'])) {
                    if(empty($filters['NRTURNO'])) {
                       unset($filters['NRTURNO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTURNO');
                    }
                }

                if(isset($filters['NRTPMOVTRANSFM'])) {
                    if(empty($filters['NRTPMOVTRANSFM'])) {
                       unset($filters['NRTPMOVTRANSFM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTPMOVTRANSFM');
                    }
                }

                if(isset($filters['NROCUPACAOM'])) {
                    if(empty($filters['NROCUPACAOM'])) {
                       unset($filters['NROCUPACAOM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NROCUPACAOM');
                    }
                }

                if(isset($filters['NRMOTIVOOCUPA'])) {
                    if(empty($filters['NRMOTIVOOCUPA'])) {
                       unset($filters['NRMOTIVOOCUPA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRMOTIVOOCUPA');
                    }
                }

                if(isset($filters['NRTIPOOCUPACAO'])) {
                    if(empty($filters['NRTIPOOCUPACAO'])) {
                       unset($filters['NRTIPOOCUPACAO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTIPOOCUPACAO');
                    }
                }

                if(isset($filters['DTINIMOVIMENT'])) {
                    if(empty($filters['DTINIMOVIMENT'])) {
                       unset($filters['DTINIMOVIMENT']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTINIMOVIMENT');
                    }
                }

                if(isset($filters['DTFIMMOVIMENT'])) {
                    if(empty($filters['DTFIMMOVIMENT'])) {
                       unset($filters['DTFIMMOVIMENT']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTFIMMOVIMENT');
                    }
                }

                
                $filters['NRTIPOSOLICITACAO'] = 29;

                return $itemsQuery;
                break;
            case 'reembolso':
                if(isset($filters['VRREEMBOLSO'])) {
                    if(empty($filters['VRREEMBOLSO'])) {
                       unset($filters['VRREEMBOLSO']); 
                    } else {
                        $filters['VRREEMBOLSO'] = str_replace(",", ".", $filters['VRREEMBOLSO']);
                        $itemsQuery .= $this->getExistsQuery('VRREEMBOLSO');
                    }
                }

                if(isset($filters['DTMESCOMPETENCIA'])) {
                    if(empty($filters['DTMESCOMPETENCIA'])) {
                       unset($filters['DTMESCOMPETENCIA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTMESCOMPETENCIA');
                    }
                }
                
                $filters['NRTIPOSOLICITACAO'] = 19;

                return $itemsQuery;
                break;
            case 'transferencia_gerencial':
                if(isset($filters['DTINIMOVIMENT'])) {
                    if(empty($filters['DTINIMOVIMENT'])) {
                       unset($filters['DTINIMOVIMENT']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTINIMOVIMENT');
                    }
                }

                if(isset($filters['NRMOTIVOOCUPA'])) {
                    if(empty($filters['NRMOTIVOOCUPA'])) {
                       unset($filters['NRMOTIVOOCUPA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRMOTIVOOCUPA');
                    }
                }

                if(isset($filters['NRESTRUTGEREN_ITEM'])) {
                    if(empty($filters['NRESTRUTGEREN_ITEM'])) {
                       unset($filters['NRESTRUTGEREN_ITEM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryForEstuturaItem('NRESTRUTGEREN_ITEM');
                    }
                }

                if(isset($filters['DTINIESCALA'])) {
                    if(empty($filters['DTINIESCALA'])) {
                       unset($filters['DTINIESCALA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('DTINIESCALA');
                    }
                }

                if(isset($filters['NRESCALATRABM'])) {
                    if(empty($filters['NRESCALATRABM'])) {
                       unset($filters['NRESCALATRABM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRESCALATRABM');
                    }
                }

                if(isset($filters['NRTURNO'])) {
                    if(empty($filters['NRTURNO'])) {
                       unset($filters['NRTURNO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTURNO');
                    }
                }

                
                $filters['NRTIPOSOLICITACAO'] = 26;

                return $itemsQuery;
                break;
            case 'transferencia_legal':
                if(isset($filters['DTINIMOVIMENT'])) {
                    if(empty($filters['DTINIMOVIMENT'])) {
                       unset($filters['DTINIMOVIMENT']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTINIMOVIMENT');
                    }
                }

                if(isset($filters['NRMOTIVOOCUPA'])) {
                    if(empty($filters['NRMOTIVOOCUPA'])) {
                       unset($filters['NRMOTIVOOCUPA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRMOTIVOOCUPA');
                    }
                }

                if(isset($filters['NRESTRUTLEGAL_ITEM'])) {
                    if(empty($filters['NRESTRUTLEGAL_ITEM'])) {
                       unset($filters['NRESTRUTLEGAL_ITEM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryForEstuturaItem('NRESTRUTLEGAL_ITEM');
                    }
                }

                if(isset($filters['NRESTRUTGEREN_ITEM'])) {
                    if(empty($filters['NRESTRUTGEREN_ITEM'])) {
                       unset($filters['NRESTRUTGEREN_ITEM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryForEstuturaItem('NRESTRUTGEREN_ITEM');
                    }
                }

                if(isset($filters['DTINIESCALA'])) {
                    if(empty($filters['DTINIESCALA'])) {
                       unset($filters['DTINIESCALA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTINIESCALA');
                    }
                }

                if(isset($filters['NRESCALATRABM'])) {
                    if(empty($filters['NRESCALATRABM'])) {
                       unset($filters['NRESCALATRABM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRESCALATRABM');
                    }
                }

                if(isset($filters['NRTURNO'])) {
                    if(empty($filters['NRTURNO'])) {
                       unset($filters['NRTURNO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTURNO');
                    }
                }
                
                $filters['NRTIPOSOLICITACAO'] = 2;

                return $itemsQuery;
                break;
            case 'substituicao':
                if(isset($filters['DTINIMOVIMENT'])) {
                    if(empty($filters['DTINIMOVIMENT'])) {
                       unset($filters['DTINIESCALA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTINIMOVIMENT');
                    }
                }

                if(isset($filters['NRMOTIVOOCUPA'])) {
                    if(empty($filters['NRMOTIVOOCUPA'])) {
                       unset($filters['NRMOTIVOOCUPA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRMOTIVOOCUPA');
                    }
                }

                if(isset($filters['NRESTRUTLEGAL_ITEM'])) {
                    if(empty($filters['NRESTRUTLEGAL_ITEM'])) {
                       unset($filters['NRESTRUTLEGAL_ITEM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryForEstuturaItem('NRESTRUTLEGAL_ITEM');
                    }
                }

                if(isset($filters['NRESTRUTGEREN_ITEM'])) {
                    if(empty($filters['NRESTRUTGEREN_ITEM'])) {
                       unset($filters['NRESTRUTGEREN_ITEM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryForEstuturaItem('NRESTRUTGEREN_ITEM');
                    }
                }

                if(isset($filters['DTINIESCALA'])) {
                    if(empty($filters['DTINIESCALA'])) {
                       unset($filters['DTINIESCALA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTINIESCALA');
                    }
                }

                if(isset($filters['NRESCALATRABM'])) {
                    if(empty($filters['NRESCALATRABM'])) {
                       unset($filters['NRESCALATRABM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRESCALATRABM');
                    }
                }

                if(isset($filters['NRTURNO'])) {
                    if(empty($filters['NRTURNO'])) {
                       unset($filters['NRTURNO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTURNO');
                    }
                }

                
                $filters['NRTIPOSOLICITACAO'] = 25;

                return $itemsQuery;
                break;
            case 'treinamento':
                if(isset($filters['IDTREINAMENTOINT'])) {
                    if(empty($filters['IDTREINAMENTOINT'])) {
                       unset($filters['IDTREINAMENTOINT']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('IDTREINAMENTOINT');
                    }
                }

                if(isset($filters['NMTREINAMENTO'])) {
                    if(empty($filters['NMTREINAMENTO'])) {
                       unset($filters['NMTREINAMENTO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('NMTREINAMENTO');
                    }
                }

                if(isset($filters['NRCURSO'])) {
                    if(empty($filters['NRCURSO'])) {
                       unset($filters['NRCURSO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRCURSO');
                    }
                }

                if(isset($filters['NRPERFACESAPROV'])) {
                    if(empty($filters['NRPERFACESAPROV'])) {
                       unset($filters['NRPERFACESAPROV']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRPERFACESAPROV');
                    }
                }

                if(isset($filters['NRMETODOTRE'])) {
                    if(empty($filters['NRMETODOTRE'])) {
                       unset($filters['NRMETODOTRE']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRMETODOTRE');
                    }
                }

                if(isset($filters['NRPERFACESEXEC'])) {
                    if(empty($filters['NRPERFACESEXEC'])) {
                       unset($filters['NRPERFACESEXEC']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRPERFACESEXEC');
                    }
                }

                if(isset($filters['DTINIPLANEJADOTRE'])) {
                    if(empty($filters['DTINIPLANEJADOTRE'])) {
                       unset($filters['DTINIPLANEJADOTRE']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTINIPLANEJADOTRE');
                    }
                }

                if(isset($filters['DTFIMPLANEJADOTRE'])) {
                    if(empty($filters['DTFIMPLANEJADOTRE'])) {
                       unset($filters['DTFIMPLANEJADOTRE']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTFIMPLANEJADOTRE');
                    }
                }

                
                $filters['NRTIPOSOLICITACAO'] = 4;

                return $itemsQuery;
                break;
            case 'vale_combustivel':
                if(isset($filters['CDTIPOALTEVC'])) {
                    if(empty($filters['CDTIPOALTEVC'])) {
                       unset($filters['CDTIPOALTEVC']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('CDTIPOALTEVC');
                    }
                }

                if(isset($filters['CDTIPOVC'])) {
                    if(empty($filters['CDTIPOVC'])) {
                       unset($filters['CDTIPOVC']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('CDTIPOVC');
                    }
                }

                if(isset($filters['QTBENEFICIO'])) {
                    if(empty($filters['QTBENEFICIO'])) {
                       unset($filters['QTBENEFICIO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('QTBENEFICIO');
                    }
                }

                if(isset($filters['NRPESSOA'])) {
                    if(empty($filters['NRPESSOA'])) {
                       unset($filters['NRPESSOA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRPESSOA');
                    }
                }

                if(isset($filters['NRBENEFICIOM'])) {
                    if(empty($filters['NRBENEFICIOM'])) {
                       unset($filters['NRBENEFICIOM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRBENEFICIOM');
                    }
                }

                if(isset($filters['DTINSCBENEFICIARIO'])) {
                    if(empty($filters['DTINSCBENEFICIARIO'])) {
                       unset($filters['DTINSCBENEFICIARIO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTINSCBENEFICIARIO');
                    }
                }

                if(isset($filters['DTCANCBENEFICIARIO'])) {
                    if(empty($filters['DTCANCBENEFICIARIO'])) {
                       unset($filters['DTCANCBENEFICIARIO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTCANCBENEFICIARIO');
                    }
                }

                
                $filters['NRTIPOSOLICITACAO'] = 21;

                return $itemsQuery;
                break;
            case 'vale_manutencao':
                if(isset($filters['CDTIPOALTEVC'])) {
                    if(empty($filters['CDTIPOALTEVC'])) {
                       unset($filters['CDTIPOALTEVC']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('CDTIPOALTEVC');
                    }
                }

                if(isset($filters['NRBENEFICIOM'])) {
                    if(empty($filters['NRBENEFICIOM'])) {
                       unset($filters['NRBENEFICIOM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRBENEFICIOM');
                    }
                }

                if(isset($filters['QTBENEFICIO'])) {
                    if(empty($filters['QTBENEFICIO'])) {
                       unset($filters['QTBENEFICIO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('QTBENEFICIO');
                    }
                }

                if(isset($filters['DTINSCBENEFICIARIO'])) {
                    if(empty($filters['DTINSCBENEFICIARIO'])) {
                       unset($filters['DTINSCBENEFICIARIO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTINSCBENEFICIARIO');
                    }
                }

                if(isset($filters['DTCANCBENEFICIARIO'])) {
                    if(empty($filters['DTCANCBENEFICIARIO'])) {
                       unset($filters['DTCANCBENEFICIARIO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTCANCBENEFICIARIO');
                    }
                }

                
                $filters['NRTIPOSOLICITACAO'] = 23;

                return $itemsQuery;
                break;
            case 'vale_transporte':
                $casesForReplica = array();

                if(isset($filters['CDTIPOALTEVT'])) {
                    if(empty($filters['CDTIPOALTEVT'])) {
                       unset($filters['CDTIPOALTEVT']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('CDTIPOALTEVT');
                    }
                }

                if(isset($filters['DTALTELINHAVINC'])) {
                    if(empty($filters['DTALTELINHAVINC'])) {
                       unset($filters['DTALTELINHAVINC']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTALTELINHAVINC');
                    }
                }

                if(isset($filters['NRLINHATRANSP'])) {
                    if(empty($filters['NRLINHATRANSP'])) {
                       unset($filters['NRLINHATRANSP']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRLINHATRANSP');
                        $casesForReplica[] = 'NRLINHATRANSP';
                    }
                }

                if(isset($filters['NRTIPOLINHAVINC'])) {
                    if(empty($filters['NRTIPOLINHAVINC'])) {
                       unset($filters['NRTIPOLINHAVINC']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTIPOLINHAVINC');
                        $casesForReplica[] = 'NRTIPOLINHAVINC';
                    }
                }

                if(isset($filters['VRTARIFA'])) {
                    if(empty($filters['VRTARIFA'])) {
                       unset($filters['VRTARIFA']); 
                    } else {
                        $filters['VRTARIFA'] = str_replace(",", ".", $filters['VRTARIFA']);
                        $itemsQuery .= $this->getExistsQuery('VRTARIFA');
                        $casesForReplica[] = 'VRTARIFA';
                    }
                }

                foreach($casesForReplica as $replica) {
                    if($replica == 'VRTARIFA') {
                        $this->getExistsQuery($replica.'_2');
                        $this->getExistsQuery($replica.'_3');
                        $this->getExistsQuery($replica.'_4');
                        $this->getExistsQuery($replica.'_5');
                        $this->getExistsQuery($replica.'_6');
                        $this->getExistsQuery($replica.'_7');
                        $this->getExistsQuery($replica.'_8');
                    } else {
                        $this->getExistsQueryWithRegexBefore($replica.'_2');
                        $this->getExistsQueryWithRegexBefore($replica.'_3');
                        $this->getExistsQueryWithRegexBefore($replica.'_4');
                        $this->getExistsQueryWithRegexBefore($replica.'_5');
                        $this->getExistsQueryWithRegexBefore($replica.'_6');
                        $this->getExistsQueryWithRegexBefore($replica.'_7');
                        $this->getExistsQueryWithRegexBefore($replica.'_8');
                    }
                }
                
                $filters['NRTIPOSOLICITACAO'] = 10;

                return $itemsQuery;
                break;
            case 'movimentacao_ocupacao_salario':
                if(isset($filters['NRTPMOVTRANSFM'])) {
                    if(empty($filters['NRTPMOVTRANSFM'])) {
                       unset($filters['NRTPMOVTRANSFM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTPMOVTRANSFM');
                    }
                }

                if(isset($filters['NRMOTIVOTRANSF'])) {
                    if(empty($filters['NRMOTIVOTRANSF'])) {
                       unset($filters['NRMOTIVOTRANSF']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRMOTIVOTRANSF');
                    }
                }

                if(isset($filters['NROCUPACAOM'])) {
                    if(empty($filters['NROCUPACAOM'])) {
                       unset($filters['NROCUPACAOM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NROCUPACAOM');
                    }
                }

                if(isset($filters['NRMOTIVOOCUPA'])) {
                    if(empty($filters['NRMOTIVOOCUPA'])) {
                       unset($filters['NRMOTIVOOCUPA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRMOTIVOOCUPA');
                    }
                }

                if(isset($filters['NRTIPOOCUPACAO'])) {
                    if(empty($filters['NRTIPOOCUPACAO'])) {
                       unset($filters['NRTIPOOCUPACAO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTIPOOCUPACAO');
                    }
                }

                if(isset($filters['VRSALARIO'])) {
                    if(empty($filters['VRSALARIO'])) {
                       unset($filters['VRSALARIO']); 
                    } else {
                        $filters['VRSALARIO'] = str_replace(",", ".", $filters['VRSALARIO']);
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('VRSALARIO');
                    }
                }

                if(isset($filters['IDTPSALARIO'])) {
                    if(empty($filters['IDTPSALARIO'])) {
                       unset($filters['IDTPSALARIO']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('IDTPSALARIO');
                    }
                }

                if(isset($filters['NRTIPOALTERA'])) {
                    if(empty($filters['NRTIPOALTERA'])) {
                       unset($filters['NRTIPOALTERA']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryWithRegexBefore('NRTIPOALTERA');
                    }
                }

                if(isset($filters['DTINIMOVIMENT'])) {
                    if(empty($filters['DTINIMOVIMENT'])) {
                       unset($filters['DTINIMOVIMENT']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTINIMOVIMENT');
                    }
                }

                if(isset($filters['DTFIMMOVIMENT'])) {
                    if(empty($filters['DTFIMMOVIMENT'])) {
                       unset($filters['DTFIMMOVIMENT']); 
                    } else {
                        $itemsQuery .= $this->getExistsQuery('DTFIMMOVIMENT');
                    }
                }

                if(isset($filters['NRESTRUTGEREN_ITEM'])) {
                    if(empty($filters['NRESTRUTGEREN_ITEM'])) {
                       unset($filters['NRESTRUTGEREN_ITEM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryForEstuturaItem('NRESTRUTGEREN_ITEM');
                    }
                }

                if(isset($filters['NRESTRUTLEGAL_ITEM'])) {
                    if(empty($filters['NRESTRUTLEGAL_ITEM'])) {
                       unset($filters['NRESTRUTLEGAL_ITEM']); 
                    } else {
                        $itemsQuery .= $this->getExistsQueryForEstuturaItem('NRESTRUTLEGAL_ITEM');
                    }
                }

                
                $filters['NRTIPOSOLICITACAO'] = 9;

                return $itemsQuery;
                break;
                
            default:
                break;
        }
    }

    private function getExistsQuery($queryKey) {
        $existsQuery = "
            AND EXISTS ( SELECT 1
                    FROM GPE_ITEMSOLICITACAO
                    WHERE NRORG         = SL.NRORG
                    AND NRSOLICITACAO = SL.NRSOLICITACAO
                    AND (DSCHAVEITEMSOLI = '$queryKey' AND DSVALORITEMSOLI = :P_$queryKey) )
        ";

        return $existsQuery;
    }

    private function getExistsQueryWithRegexBefore($queryKey) {
        $existsQuery = "
            AND EXISTS ( SELECT 1
                    FROM GPE_ITEMSOLICITACAO
                    WHERE NRORG         = SL.NRORG
                    AND NRSOLICITACAO = SL.NRSOLICITACAO
                    AND (DSCHAVEITEMSOLI = '$queryKey' AND TRIM( REGEXP_SUBSTR( DSVALORITEMSOLI, '(.*)-', 1, 1, 'i', 1 ) ) = :P_$queryKey) )
        ";

        return $existsQuery;
    }

    private function getExistsQueryWithRegexAfter($queryKey) {
        $existsQuery = "
            AND EXISTS ( SELECT 1
                    FROM GPE_ITEMSOLICITACAO
                    WHERE NRORG         = SL.NRORG
                    AND NRSOLICITACAO = SL.NRSOLICITACAO
                    AND (DSCHAVEITEMSOLI = '$queryKey' AND TRIM( REGEXP_SUBSTR( DSVALORITEMSOLI, '-(.*)', 1, 1, 'i', 1 ) ) = :P_$queryKey) )
        ";

        return $existsQuery;
    }

    private function getExistsQueryForEstuturaItem($queryKey) {
        $queryKeyAux = str_replace('_ITEM', '', $queryKey);
        $existsQuery = "
            AND EXISTS ( SELECT 1
                    FROM GPE_ITEMSOLICITACAO
                    WHERE NRORG         = SL.NRORG
                    AND NRSOLICITACAO = SL.NRSOLICITACAO
                    AND (DSCHAVEITEMSOLI = '$queryKeyAux' AND TRIM( REGEXP_SUBSTR( DSVALORITEMSOLI, '-(.*)', 1, 1, 'i', 1 ) ) = :P_$queryKeyAux) )
        ";

        return $existsQuery;
    }
}
